rm(list=ls(all=TRUE))  # remove all existing items from the workspace
library(dplyr)
library(reshape2)

#You can install the released version of TraitEnvMLMWA from github by
#invoking the R-code within an R-console:
#
#install.packages("remotes")
#remotes::install_github("CajoterBraak/TraitEnvMLMWA")
#
library(TraitEnvMLMWA)
source("expand4glmm.r")

# function definition -----------------------------------------------------
TabEnvTrait <- function(Env,L, Traits, Envname, Traitname, Effort = rep(1,nrow(L)), Effort.reference = 1){
  # numbers per habitat class
  Env <- data.frame(Env[,Envname]); names(Env)<- Envname
  Traits <-  data.frame(Traits[,Traitname]); names(Traits)<- Traitname
  TotH <- tapply(rowSums(L), INDEX= Env[[Envname]], FUN = sum)
  TotEff <- tapply(Effort, INDEX= Env[[Envname]], FUN = sum)
  CountsPerEnvClass <- TotH/TotEff
  
  TotTrait <- tapply(colSums(L), INDEX= Traits[[Traitname]], FUN = sum)
  CountTrait <- TotTrait/mean(Effort)/nrow(L)
  CountTrait <- c(CountTrait, total = sum(CountTrait))
  
  # Number of beetles in crossclass of env and trait using L ------------------
  
  obj <- make_obj_for_traitenv(Env, L, Traits )
  mydat <- expand4glmm(obj, with_interactions = FALSE)
  form <- as.formula( paste("y~",Envname,"+",Traitname, sep =""))
  tabCounts <- xtabs(form, data = mydat)
  
  #tabCounts
  #dim(tabCounts)
  #length(TotEff)
  CountPerDay <- sweep(tabCounts, MARGIN= 1, STATS= TotEff[dimnames(tabCounts)[[1]]], FUN= "/")[levels(Env[[names(dimnames(tabCounts)[1])]]),
                                                                                                levels(Traits[[names(dimnames(tabCounts)[2])]])]
  
  
  CountPerDayMarg <- addmargins(CountPerDay, margin = seq_along(dim(CountPerDay)), FUN = sum, quiet = FALSE)
  
  CountPerDayRowwise <- sweep(CountPerDayMarg, MARGIN = 1, STATS = CountPerDayMarg[,ncol(CountPerDayMarg)], FUN= "/" )# 100 each row
  CountPerDayRowwise[,"sum"] <- Effort.reference*CountPerDayMarg[,ncol(CountPerDayMarg)]
  CountPerDayRowwise["sum",] <- Effort.reference*CountTrait 
  
  colnames(CountPerDayRowwise)[ncol(CountPerDayRowwise)] <- "250d-total"
  rownames(CountPerDayRowwise)[nrow(CountPerDayRowwise)] <- "250day-total"
  
  #round(CountPerDayRowwise, 2)
  out <- list(CountPerDayRowwise = CountPerDayRowwise, 
              CountPerDayMarg= CountPerDayMarg,
              CountPerDay = CountPerDay,
              CountTrait=CountTrait,
              EffortPerEnvClass=TotEff,
              CountsPerEnvClass = CountsPerEnvClass,
              tabCounts = tabCounts)
  return(out)
}




# get data set ----------------------------------------------------------

Abun0<- read.csv("data/AbundancePF_reordered_imputed.csv", skip = 1)
#View(Abun0)
Env0<- read.csv("data/EnvironmentPF_reordered_imputed.csv", skip = 1)
#View(Env0)
names(Env0)
nlevels(factor(Env0$Plot)) -nlevels(factor(Env0$Transect))
nrow(Env0) -nlevels(factor(Env0$Plot)) 
Trait0<- read.csv("data/TraitsPF_with_dummies.csv",skip=1)
#View(Trait0)

#Env0$Soil <- log(Env0$Soil+1)
Effort0 <- Env0$EffrDays
range(Effort0,na.rm = TRUE)
id <- which(is.na(Effort0))
Env0$X[id]
Effort <- Env0$EffrDays

L0 <- Abun0[,-(1:2)]
names(L0)
names(Env0)
#E <- as.matrix(Env0[,-c(1:5, 7, 26:30)])
Plot <- Env0$Plot
Transect <- Env0$Transect
Sector <- Env0$Sector
# permutation of plots along transects is only possible for balanced data, therefore we impute the missings abundance and effort
# impute effort to the mean of the other pitfalls (hm: generates extra data; aim to allow Abundance/Effort)
mean_effort_per_plot <- tapply(Effort0, Plot, function(x)mean(x,na.rm=TRUE))
impute_effort <- mean_effort_per_plot[Plot[id]]
Effort[id] <- impute_effort
Env0$Effort <- Effort

names(Trait0)
traits <- Trait0[,-c(1:2)]
names(traits)

Env <- Env0[-id, ]
L <- L0[-id,]
Effort <- Env$EffrDays

# Number of beetles in Habitat using Env0$Habitat and L0 ------------------


# Define suitable levels of variables and orders thereof ------------------

# from 0 1 to reverse 1 2 with 1 == 1 of 0 1

# in Env
Env$Habitat <- factor(2*(-0.5*Env$HayMeadw+1))
levels(Env$Habitat) <- c("Hay meadow", "Other habitat")
Env$pH <- cut(Env$Ph, breaks = c(0,4,5,6,10))
levels(Env$pH) <- c("<4", "4-5", "5-6", ">6")

Env$Elevation <- cut(Env$Elevatio, breaks = c(0,1500,2000,2500,3000))
levels(Env$Elevation) <- c("<1500", "1500-2000", "2000-2500", ">2500" )

Env$Canopy <- factor(ifelse(Env$Canopy >0, "Canopy", "No canopy"))


# in Trait
traits$Wing <- factor(2*(-0.5*as.numeric(traits$Brachypt) +1 ))
levels(traits$Wing) <- c("Brachypterous","Other wing types")
traits$Diet <- factor(2*(-0.5*as.numeric(traits$SpecPred) +1 ))
levels(traits$Diet) <- c("Specialized predator","Other diet types")
traits$bodylength <- log(traits$BLength)
exp(c(1.7,2.5))
summary(traits$BLength)
summary(cut(traits$bodylength,breaks= 3))

traits$bodylength <- cut(traits$BLength, breaks = c(0,7,10,3000))
summary(traits$bodylength)
levels(traits$bodylength) <- c("<7","7-10", ">10")

Env$Elevation <- cut(Env$Elevatio, breaks = c(0,1500,2000,2500,3000))
levels(Env$Elevation) <- c("<1500", "1500-2000", "2000-2500", ">2500" )


summary(Env)
summary(traits)






# Start calculations ------------------------------------------------------




# Apply function-------------------------------------------------------------------

dim(L)
dim(Env)
dim(traits)
length(Effort)
#traits$Wing
names(Env)
names(traits)
EnvFactornames <- c("Habitat", "pH","Elevation","Canopy")
summary(Env[,EnvFactornames])
TraitFactornames <- c( "Wing", "Diet", "bodylength")
summary(traits[,TraitFactornames])


Effort.reference <- 250

out.list <- list()
tab.list <- list()


for (idt in seq_along(TraitFactornames)){ 

out<-   TabEnvTrait(Env= Env,L= L, Traits = traits, Effort = Effort,
                    Envname = EnvFactornames[length(EnvFactornames)],
                    Traitname = TraitFactornames[idt], Effort.reference = Effort.reference )
tab <-out$CountPerDayRowwise# last table

for (Envname in rev(EnvFactornames)[-1]){
  out<-   TabEnvTrait(Env= Env,L= L, Traits = traits, Effort = Effort,
                      Envname = Envname,Traitname = TraitFactornames[idt], Effort.reference = Effort.reference )
  tab <- rbind( out$CountPerDayRowwise[-nrow(out$CountPerDayRowwise),], tab)
}
out.list[[idt]] <- out
tab.list[[idt]] <- tab
print(TraitFactornames[idt])
print(round(tab, 2))
}
names(tab.list) <- TraitFactornames



write.csv(tab.list[[1]], file="Brachcount.csv")
write.csv(tab.list[[2]], file="SpecPredcount.csv")
write.csv(tab.list[[3]], file="BodyLengthcount.csv")

out.list
tab.list




